git/list[1] front-page[2] threads[3] people[4] search[5] about
 

[Outreachy PATCH RFC 2/3] environment: stop using core.sparseCheckout globally

From
Olamide Caleb Bello <belkid98@gmail.com>
Date
Jan 12, 2026, 12:59 UTC
Message-ID
<87b86239ff31cef5ebb6264d630b9072132c490f.1768217572.git.belkid98@gmail.com>
In-Reply-To
<cover.1768217572.git.belkid98@gmail.com>

The config value `core.sparseCheckout` is parsed in `git_default_core_config()` and stored globally in `core_appy_sparse_checkout`. This could cause unintended behaviours when different Git repositories running in the same process access this variable.

Move the parsed value into `struct config_values` which holds all the values parsed by `git_default_config()` and can be accessed per repo via `git_default_config()`. This would mean we do not need to remove code from `git_default_core_config()`, thereby retaining current behaviours.

Suggested-by: Phillip Wood <phillip.wood123@gmail.com>
Mentored-by: Christian Couder <christian.couder@gmail.com>
Mentored-by: Usman Akinyemi <usmanakinyemi202@gmail.com>
Signed-off-by: Olamide Caleb Bello <belkid98@gmail.com>
---
 builtin/backfill.c        |  3 +--
 builtin/clone.c           |  2 +-
 builtin/mv.c              |  2 +-
 builtin/sparse-checkout.c | 22 +++++++++++-----------
 builtin/worktree.c        |  2 +-
 config.h                  |  1 +
 dir.c                     |  2 +-
 environment.c             |  3 +--
 environment.h             |  1 -
 sparse-index.c            |  4 ++--
 unpack-trees.c            |  2 +-
 wt-status.c               |  3 ++-
 12 files changed, 23 insertions(+), 24 deletions(-)
diff --git a/builtin/backfill.c b/builtin/backfill.c
index e80fc1b694..7745aa6bf9 100644
--- a/builtin/backfill.c
+++ b/builtin/backfill.c
@@ -1,4 +1,3 @@
-/* We need this macro to access core_apply_sparse_checkout */
 #define USE_THE_REPOSITORY_VARIABLE
 
 #include "builtin.h"
@@ -139,7 +138,7 @@ int cmd_backfill(int argc, const char **argv, const char *prefix, struct reposit
 	repo_config(repo, git_default_config, NULL);
 
 	if (ctx.sparse < 0)
-		ctx.sparse = core_apply_sparse_checkout;
+		ctx.sparse = repo->cfg_values->sparse_checkout;
 
 	result = do_backfill(&ctx);
 	backfill_context_clear(&ctx);
diff --git a/builtin/clone.c b/builtin/clone.c
index b19b302b06..568e76a135 100644
--- a/builtin/clone.c
+++ b/builtin/clone.c
@@ -623,7 +623,7 @@ static int git_sparse_checkout_init(const char *repo)
 	 * We must apply the setting in the current process
 	 * for the later checkout to use the sparse-checkout file.
 	 */
-	core_apply_sparse_checkout = 1;
+	the_repository->cfg_values->sparse_checkout = 1;
 
 	cmd.git_cmd = 1;
 	if (run_command(&cmd)) {
diff --git a/builtin/mv.c b/builtin/mv.c
index d43925097b..e95c1dfb91 100644
--- a/builtin/mv.c
+++ b/builtin/mv.c
@@ -572,7 +572,7 @@ int cmd_mv(int argc,
 		rename_index_entry_at(the_repository->index, pos, dst);
 
 		if (ignore_sparse &&
-		    core_apply_sparse_checkout &&
+		    the_repository->cfg_values->sparse_checkout &&
 		    core_sparse_checkout_cone) {
 			/*
 			 * NEEDSWORK: we are *not* paying attention to
diff --git a/builtin/worktree.c b/builtin/worktree.c
index fbdaf2eb2e..921b4a8ca2 100644
--- a/builtin/worktree.c
+++ b/builtin/worktree.c
@@ -536,7 +536,7 @@ static int add_worktree(const char *path, const char *refname,
 	 * If the current worktree has sparse-checkout enabled, then copy
 	 * the sparse-checkout patterns from the current worktree.
 	 */
-	if (core_apply_sparse_checkout)
+	if (the_repository->cfg_values->sparse_checkout)
 		copy_sparse_checkout(sb_repo.buf);
 
 	/*
diff --git a/config.h b/config.h
index 1652d315e2..21d65c440e 100644
--- a/config.h
+++ b/config.h
@@ -140,6 +140,7 @@ struct config_context {
 struct config_values {
 	/* core config values */
 	char *attributes_file_path;
+	int sparse_checkout;
 
 };
 #define CONFIG_CONTEXT_INIT { 0 }
diff --git a/dir.c b/dir.c
index b00821f294..d23f057f95 100644
--- a/dir.c
+++ b/dir.c
@@ -1551,7 +1551,7 @@ enum pattern_match_result path_matches_pattern_list(
 
 int init_sparse_checkout_patterns(struct index_state *istate)
 {
-	if (!core_apply_sparse_checkout)
+	if (!istate->repo->cfg_values->sparse_checkout)
 		return 1;
 	if (istate->sparse_checkout_patterns)
 		return 0;
diff --git a/environment.c b/environment.c
index d633b0405b..15b63cdc4a 100644
--- a/environment.c
+++ b/environment.c
@@ -74,7 +74,6 @@ enum push_default_type push_default = PUSH_DEFAULT_UNSPECIFIED;
 #endif
 enum object_creation_mode object_creation_mode = OBJECT_CREATION_MODE;
 int grafts_keep_true_parents;
-int core_apply_sparse_checkout;
 int core_sparse_checkout_cone;
 int sparse_expect_files_outside_of_patterns;
 int precomposed_unicode = -1; /* see probe_utf8_pathname_composition() */
@@ -546,7 +545,7 @@ static int git_default_core_config(const char *var, const char *value,
 	}
 
 	if (!strcmp(var, "core.sparsecheckout")) {
-		core_apply_sparse_checkout = git_config_bool(var, value);
+		cfg->sparse_checkout = git_config_bool(var, value);
 		return 0;
 	}
 
diff --git a/environment.h b/environment.h
index 3512a7072e..60d5933cc2 100644
--- a/environment.h
+++ b/environment.h
@@ -161,7 +161,6 @@ extern int precomposed_unicode;
 extern int protect_hfs;
 extern int protect_ntfs;
 
-extern int core_apply_sparse_checkout;
 extern int core_sparse_checkout_cone;
 extern int sparse_expect_files_outside_of_patterns;
 
diff --git a/sparse-index.c b/sparse-index.c
index 76f90da5f5..14f3f07fe9 100644
--- a/sparse-index.c
+++ b/sparse-index.c
@@ -152,7 +152,7 @@ static int index_has_unmerged_entries(struct index_state *istate)
 
 int is_sparse_index_allowed(struct index_state *istate, int flags)
 {
-	if (!core_apply_sparse_checkout || !core_sparse_checkout_cone)
+	if (!istate->repo->cfg_values->sparse_checkout || !core_sparse_checkout_cone)
 		return 0;
 
 	if (!(flags & SPARSE_INDEX_MEMORY_ONLY)) {
@@ -670,7 +670,7 @@ static void clear_skip_worktree_from_present_files_full(struct index_state *ista
 
 void clear_skip_worktree_from_present_files(struct index_state *istate)
 {
-	if (!core_apply_sparse_checkout ||
+	if (!istate->repo->cfg_values->sparse_checkout ||
 	    sparse_expect_files_outside_of_patterns)
 		return;
 
diff --git a/unpack-trees.c b/unpack-trees.c
index f38c761ab9..ffab104bfb 100644
--- a/unpack-trees.c
+++ b/unpack-trees.c
@@ -1924,7 +1924,7 @@ int unpack_trees(unsigned len, struct tree_desc *t, struct unpack_trees_options
 	if (o->prefix)
 		update_sparsity_for_prefix(o->prefix, o->src_index);
 
-	if (!core_apply_sparse_checkout || !o->update)
+	if (!repo->cfg_values->sparse_checkout || !o->update)
 		o->skip_sparse_checkout = 1;
 	if (!o->skip_sparse_checkout) {
 		memset(&pl, 0, sizeof(pl));
diff --git a/wt-status.c b/wt-status.c
index e12adb26b9..de527bd620 100644
--- a/wt-status.c
+++ b/wt-status.c
@@ -35,6 +35,7 @@
 #include "lockfile.h"
 #include "sequencer.h"
 #include "fsmonitor-settings.h"
+#include "config.h"
 
 #define AB_DELAY_WARNING_IN_MS (2 * 1000)
 #define UF_DELAY_WARNING_IN_MS (2 * 1000)
@@ -1764,7 +1765,7 @@ static void wt_status_check_sparse_checkout(struct repository *r,
 	int skip_worktree = 0;
 	int i;
 
-	if (!core_apply_sparse_checkout || r->index->cache_nr == 0) {
+	if (!r->cfg_values->sparse_checkout || r->index->cache_nr == 0) {
 		/*
 		 * Don't compute percentage of checked out files if we
 		 * aren't in a sparse checkout or would get division by 0.
diff --git a/builtin/sparse-checkout.c b/builtin/sparse-checkout.c
index 15d51e60a8..0eb81b2188 100644
--- a/builtin/sparse-checkout.c
+++ b/builtin/sparse-checkout.c
@@ -63,7 +63,7 @@ static int sparse_checkout_list(int argc, const char **argv, const char *prefix,
 	int res;
 
 	setup_work_tree();
-	if (!core_apply_sparse_checkout)
+	if (!the_repository->cfg_values->sparse_checkout)
 		die(_("this worktree is not sparse"));
 
 	argc = parse_options(argc, argv, prefix,
@@ -400,11 +400,11 @@ static int set_config(struct repository *repo,
 
 static enum sparse_checkout_mode update_cone_mode(int *cone_mode) {
 	/* If not specified, use previous definition of cone mode */
-	if (*cone_mode == -1 && core_apply_sparse_checkout)
+	if (*cone_mode == -1 && the_repository->cfg_values->sparse_checkout)
 		*cone_mode = core_sparse_checkout_cone;
 
 	/* Set cone/non-cone mode appropriately */
-	core_apply_sparse_checkout = 1;
+	the_repository->cfg_values->sparse_checkout = 1;
 	if (*cone_mode == 1 || *cone_mode == -1) {
 		core_sparse_checkout_cone = 1;
 		return MODE_CONE_PATTERNS;
@@ -418,7 +418,7 @@ static int update_modes(struct repository *repo, int *cone_mode, int *sparse_ind
 	int mode, record_mode;
 
 	/* Determine if we need to record the mode; ensure sparse checkout on */
-	record_mode = (*cone_mode != -1) || !core_apply_sparse_checkout;
+	record_mode = (*cone_mode != -1) || !repo->cfg_values->sparse_checkout;
 
 	mode = update_cone_mode(cone_mode);
 	if (record_mode && set_config(repo, mode))
@@ -699,9 +699,9 @@ static int modify_pattern_list(struct repository *repo,
 		break;
 	}
 
-	if (!core_apply_sparse_checkout) {
+	if (!repo->cfg_values->sparse_checkout) {
 		set_config(repo, MODE_ALL_PATTERNS);
-		core_apply_sparse_checkout = 1;
+		repo->cfg_values->sparse_checkout = 1;
 		changed_config = 1;
 	}
 
@@ -798,7 +798,7 @@ static int sparse_checkout_add(int argc, const char **argv, const char *prefix,
 	int ret;
 
 	setup_work_tree();
-	if (!core_apply_sparse_checkout)
+	if (!repo->cfg_values->sparse_checkout)
 		die(_("no sparse-checkout to add to"));
 
 	repo_read_index(repo);
@@ -907,7 +907,7 @@ static int sparse_checkout_reapply(int argc, const char **argv,
 	};
 
 	setup_work_tree();
-	if (!core_apply_sparse_checkout)
+	if (!repo->cfg_values->sparse_checkout)
 		die(_("must be in a sparse-checkout to reapply sparsity patterns"));
 
 	reapply_opts.cone_mode = -1;
@@ -969,7 +969,7 @@ static int sparse_checkout_clean(int argc, const char **argv,
 	};
 
 	setup_work_tree();
-	if (!core_apply_sparse_checkout)
+	if (!repo->cfg_values->sparse_checkout)
 		die(_("must be in a sparse-checkout to clean directories"));
 	if (!core_sparse_checkout_cone)
 		die(_("must be in a cone-mode sparse-checkout to clean directories"));
@@ -1035,7 +1035,7 @@ static int sparse_checkout_disable(int argc, const char **argv,
 	struct pattern_list pl;
 
 	/*
 	 * We do not exit early if !core_apply_sparse_checkout; due to the
 	 * ability for users to manually muck things up between
 	 *   direct editing of .git/info/sparse-checkout
 	 *   running read-tree -m u HEAD or update-index --skip-worktree
@@ -1061,7 +1061,7 @@ static int sparse_checkout_disable(int argc, const char **argv,
 	hashmap_init(&pl.recursive_hashmap, pl_hashmap_cmp, NULL, 0);
 	hashmap_init(&pl.parent_hashmap, pl_hashmap_cmp, NULL, 0);
 	pl.use_cone_patterns = 0;
-	core_apply_sparse_checkout = 1;
+	repo->cfg_values->sparse_checkout = 1;
 
 	add_pattern("/*", empty_base, 0, &pl, 0);
 
@@ -1035,7 +1035,7 @@ static int sparse_checkout_disable(int argc, const char **argv,
 
 	/*
-	 * We do not exit early if !core_apply_sparse_checkout; due to the
+	 * We do not exit early if !repo->cfg_values->sparse_checkout; due to the
 	 * ability for users to manually muck things up between
 	 *   direct editing of .git/info/sparse-checkout
-- 
2.34.1
Previous: Bello OlamideNext: Olamide Caleb Bello
Message 5 of 71 in “store git_default_config() parsed values in new config struct”
  1. 0/3 store git_default_config() parsed values in new config structOlamide Caleb Bello, Jan 12, 2026
  2. 1/3 environment: stop storing `core.attributesFile` globallyOlamide Caleb Bello, Jan 12, 2026
  3. Phillip WoodJan 12, 2026
  4. Bello OlamideJan 12, 2026
  5. 2/3 environment: stop using core.sparseCheckout globallyOlamide Caleb Bello, Jan 12, 2026
  6. 3/3 environment: move "branch.autoSetupMerge" into `struct config_values`Olamide Caleb Bello, Jan 12, 2026
  7. 0/3 store git_default_config() parsed values in new config structOlamide Caleb Bello, Jan 13, 2026
  8. 1/3 environment: stop storing `core.attributesFile` globallyOlamide Caleb Bello, Jan 13, 2026
  9. Junio C HamanoJan 13, 2026
  10. Bello OlamideJan 14, 2026
  11. 2/3 environment: environment: stop using core.sparseCheckout globallyOlamide Caleb Bello, Jan 13, 2026
  12. Junio C HamanoJan 13, 2026
  13. Bello OlamideJan 14, 2026
  14. 3/3 environment: move "branch.autoSetupMerge" into `struct repo_config_values`Olamide Caleb Bello, Jan 13, 2026
  15. Junio C HamanoJan 13, 2026
  16. Bello OlamideJan 14, 2026
  17. Bello OlamideJan 15, 2026
  18. 0/3 store repo specific config values in new `struct repo_config_values`Olamide Caleb Bello, Jan 17, 2026
  19. 1/3 environment: stop storing `core.attributesFile` globallyOlamide Caleb Bello, Jan 17, 2026
  20. Toon ClaesJan 22, 2026
  21. Bello OlamideJan 22, 2026
  22. Phillip WoodJan 22, 2026
  23. Bello OlamideJan 22, 2026
  24. 2/3 environment: environment: stop using core.sparseCheckout globallyOlamide Caleb Bello, Jan 17, 2026
  25. Toon ClaesJan 22, 2026
  26. Bello OlamideJan 22, 2026
  27. Phillip WoodJan 22, 2026
  28. Bello OlamideJan 22, 2026
  29. Phillip WoodJan 23, 2026
  30. Bello OlamideJan 23, 2026
  31. 3/3 environment: move "branch.autoSetupMerge" into `struct repo_config_values`Olamide Caleb Bello, Jan 17, 2026
  32. Phillip WoodJan 22, 2026
  33. Bello OlamideJan 22, 2026
  34. Bello OlamideJan 20, 2026
  35. 0/3 store repo specific config values in new `struct repo_config_values`Olamide Caleb Bello, Jan 24, 2026
  36. 1/3 environment: stop storing `core.attributesFile` globallyOlamide Caleb Bello, Jan 24, 2026
  37. 2/3 environment: stop using core.sparseCheckout globallyOlamide Caleb Bello, Jan 24, 2026
  38. 3/3 environment: move "branch.autoSetupMerge" into `struct repo_config_values`Olamide Caleb Bello, Jan 24, 2026
  39. 0/3 store repo specific config values in new `struct repo_config_values`Olamide Caleb Bello, Jan 24, 2026
  40. 1/3 environment: stop storing `core.attributesFile` globallyOlamide Caleb Bello, Jan 24, 2026
  41. Junio C HamanoJan 29, 2026
  42. 2/3 environment: stop using core.sparseCheckout globallyOlamide Caleb Bello, Jan 24, 2026
  43. Junio C HamanoJan 29, 2026
  44. 3/3 environment: move "branch.autoSetupMerge" into `struct repo_config_values`Olamide Caleb Bello, Jan 24, 2026
  45. Junio C HamanoJan 29, 2026
  46. Junio C HamanoJan 30, 2026
  47. Junio C HamanoJan 30, 2026
  48. Bello OlamideJan 29, 2026
  49. 0/3 store repo specific config values in new `struct repo_config_values`Olamide Caleb Bello, Feb 3, 2026
  50. 1/3 environment: stop storing `core.attributesFile` globallyOlamide Caleb Bello, Feb 3, 2026
  51. Phillip WoodFeb 4, 2026
  52. Bello OlamideFeb 9, 2026
  53. Junio C HamanoFeb 7, 2026
  54. Phillip WoodFeb 8, 2026
  55. Bello OlamideFeb 9, 2026
  56. Bello OlamideFeb 10, 2026
  57. 2/3 environment: stop using core.sparseCheckout globallyOlamide Caleb Bello, Feb 3, 2026
  58. Phillip WoodFeb 4, 2026
  59. 3/3 environment: move "branch.autoSetupMerge" into `struct repo_config_values`Olamide Caleb Bello, Feb 3, 2026
  60. Phillip WoodFeb 4, 2026
  61. 0/3 store repo specific config values in new `struct repo_config_values`Olamide Caleb Bello, Feb 16, 2026
  62. 1/3 environment: stop storing `core.attributesFile` globallyOlamide Caleb Bello, Feb 16, 2026
  63. 2/3 environment: stop using core.sparseCheckout globallyOlamide Caleb Bello, Feb 16, 2026
  64. Christian CouderFeb 26, 2026
  65. Junio C HamanoFeb 26, 2026
  66. Bello OlamideFeb 26, 2026
  67. 3/3 environment: move "branch.autoSetupMerge" into `struct repo_config_values`Olamide Caleb Bello, Feb 16, 2026
  68. Junio C HamanoFeb 17, 2026
  69. Bello OlamideFeb 18, 2026
  70. Christian CouderFeb 26, 2026
  71. Junio C HamanoFeb 26, 2026

Read the whole thread, see it on lore, or plain text.

$ cat FOOTERMessages come from the public archive at lore.kernel.org/git, fetched every hour. The front page is chosen and written each morning by an AI editor and can be wrong; the threads themselves are the record. About and API. For agents: an MCP server at https://gitlist.dev/mcp, and any thread, story or person page as Markdown by adding .md to its URL (or sending Accept: text/markdown). Details in /llms.txt.